@@ -1,4 +1,4 @@
-/*      $Id: irrecord.c,v 5.69 2008/02/14 20:42:56 lirc Exp $      */
+/*      $Id: irrecord.c,v 5.94 2009/04/28 16:24:36 lirc Exp $      */
 
 /****************************************************************************
  ** irrecord.c **************************************************************
@@ -51,6 +51,8 @@
 #include "dump_config.h"
 #include "ir_remote.h"
 #include "config_file.h"
+#include "receive.h"
+#include "input_map.h"
 
 void flushhw(void);
 int resethw(void);
@@ -60,28 +62,29 @@ int get_toggle_bit_mask(struct ir_remote *remote);
 void set_toggle_bit_mask(struct ir_remote *remote,ir_code xor);
 void get_pre_data(struct ir_remote *remote);
 void get_post_data(struct ir_remote *remote);
-#ifdef DEBUG
 typedef void (*remote_func)(struct ir_remote *remotes);
 
 void for_each_remote(struct ir_remote *remotes, remote_func func);
+void analyse_remote(struct ir_remote *raw_data);
+#ifdef DEBUG
 void remove_pre_data(struct ir_remote *remote);
 void remove_post_data(struct ir_remote *remote);
 void invert_data(struct ir_remote *remote);
 void remove_trail(struct ir_remote *remote);
 #endif
-int get_lengths(struct ir_remote *remote,int force);
+int get_lengths(struct ir_remote *remote, int force, int interactive);
 struct lengths *new_length(lirc_t length);
 int add_length(struct lengths **first,lirc_t length);
-void free_lengths(struct lengths *first);
+void free_lengths(struct lengths **firstp);
 void merge_lengths(struct lengths *first);
-void get_scheme(struct ir_remote *remote);
+void get_scheme(struct ir_remote *remote, int interactive);
 struct lengths *get_max_length(struct lengths *first,unsigned int *sump);
 void unlink_length(struct lengths **first,struct lengths *remove);
-int get_trail_length(struct ir_remote *remote);
-int get_lead_length(struct ir_remote *remote);
-int get_repeat_length(struct ir_remote *remote);
-int get_header_length(struct ir_remote *remote);
-int get_data_length(struct ir_remote *remote);
+int get_trail_length(struct ir_remote *remote, int interactive);
+int get_lead_length(struct ir_remote *remote, int interactive);
+int get_repeat_length(struct ir_remote *remote, int interactive);
+int get_header_length(struct ir_remote *remote, int interactive);
+int get_data_length(struct ir_remote *remote, int interactive);
 int get_gap_length(struct ir_remote *remote);
 void fprint_copyright(FILE *fout);
 
@@ -94,7 +97,7 @@ const char *usage="Usage: %s [options] file\n";
 struct ir_remote remote;
 struct ir_ncode ncode;
 
-#define IRRECORD_VERSION "0.5"
+#define IRRECORD_VERSION "$Revision: 5.94 $"
 #define BUTTON 80+1
 #define RETRIES 10
 
@@ -108,8 +111,8 @@ struct ir_ncode ncode;
 
 lirc_t signals[MAX_SIGNALS];
 
-#define AEPS (hw.resolution>100 ? hw.resolution:100)
-#define EPS 30
+unsigned int eps = 30;
+lirc_t aeps = 100;
 
 /* some threshold values */
 
@@ -118,7 +121,8 @@ lirc_t signals[MAX_SIGNALS];
 #define TH_REPEAT      90
 #define TH_TRAIL       90
 #define TH_LEAD        90
-#define TH_IS_BIT      15
+#define TH_IS_BIT      10
+#define TH_RC6_SIGNAL 550
 
 #define MIN_GAP  20000
 #define MAX_GAP 100000
@@ -134,6 +138,128 @@ FILE *lf=NULL;
 char *hostname="";
 int daemonized=0;
 
+struct ir_remote *emulation_data;
+struct ir_ncode *next_code = NULL;
+struct ir_ncode *current_code = NULL;
+int current_index = 0;
+int current_rep = 0;
+
+lirc_t emulation_readdata(lirc_t timeout)
+{
+	static lirc_t sum = 0;
+	lirc_t data = 0;
+	
+	if(current_code == NULL)
+	{
+		data = 1000000;
+		if(next_code)
+		{
+			current_code = next_code;
+		}
+		else
+		{
+			current_code = emulation_data->codes;
+		}
+		current_rep = 0;
+		sum = 0;
+	}
+	else
+	{
+		if(current_code->name == NULL)
+		{
+			fprintf(stderr, "%s: %s no data found\n",
+				progname, emulation_data->name);
+			data = 0;
+		}
+		if(current_index >= current_code->length)
+		{
+			if(next_code)
+			{
+				current_code = next_code;
+			}
+			else
+			{
+				current_rep++;
+				if(current_rep > 2)
+				{
+					current_code++;
+					current_rep = 0;
+					data = 1000000;
+				}
+			}
+			current_index = 0;
+			if(current_code->name == NULL)
+			{
+				current_code = NULL;
+				return emulation_readdata(timeout);
+			}
+			if(data == 0)
+			{
+				if(is_const(emulation_data))
+				{
+					data = emulation_data->gap - sum;
+				}
+				else
+				{
+					data = emulation_data->gap;
+				}
+			}
+			
+			sum = 0;
+		}
+		else
+		{
+			data = current_code->signals[current_index];
+			if((current_index % 2) == 0)
+			{
+				data |= PULSE_BIT;
+			}
+			current_index++;
+			sum += data&PULSE_MASK;
+		}
+	}
+	/*
+	printf("delivering: %c%u\n", data&PULSE_BIT ? 'p':'s',
+	       data&PULSE_MASK);
+	*/
+	return data;
+}
+
+struct hardware hw_emulation=
+{
+	"/dev/null",        /* default device */
+	-1,                 /* fd */
+	LIRC_CAN_REC_MODE2, /* features */
+	0,                  /* send_mode */
+	LIRC_MODE_MODE2,    /* rec_mode */
+	0,                  /* code_length */
+	NULL,               /* init_func */
+	NULL,               /* config_func */
+	NULL,               /* deinit_func */
+	NULL,               /* send_func */
+	NULL,               /* rec_func */
+	NULL,               /* decode_func */
+	NULL,               /* ioctl_func */
+	emulation_readdata,
+	"emulation"
+};
+
+static int iprintf(int interactive, char *format_str, ...)
+{
+	va_list ap;
+	int ret = 0;
+	
+#ifndef DEBUG
+	if(interactive)
+#endif
+	{
+		va_start(ap,format_str);
+		ret = vfprintf(stdout, format_str, ap);
+		va_end(ap);
+	}
+	return ret;
+}
+
 void logprintf(int prio,char *format_str, ...)
 {
 	va_list ap;  
@@ -191,9 +317,13 @@ int main(int argc,char **argv)
 	int repeat_flag;
 	lirc_t min_remaining_gap, max_remaining_gap;
 	int force;
+	int disable_namespace = 0;
 	int retries;
+	int no_data = 0;
 	struct ir_remote *remotes=NULL;
 	char *device=NULL;
+	int using_template = 0;
+	int analyse = 0;
 #ifdef DEBUG
 	int get_pre=0,get_post=0,test=0,invert=0,trail=0;
 #endif
@@ -208,9 +338,12 @@ int main(int argc,char **argv)
 		{
 			{"help",no_argument,NULL,'h'},
 			{"version",no_argument,NULL,'v'},
+			{"analyse",no_argument,NULL,'a'},			
 			{"device",required_argument,NULL,'d'},
 			{"driver",required_argument,NULL,'H'},
 			{"force",no_argument,NULL,'f'},
+			{"disable-namespace",no_argument,NULL,'n'},
+			{"list-namespace",no_argument,NULL,'l'},
 #ifdef DEBUG
 			{"pre",no_argument,NULL,'p'},
 			{"post",no_argument,NULL,'P'},
@@ -221,9 +354,9 @@ int main(int argc,char **argv)
 			{0, 0, 0, 0}
 		};
 #ifdef DEBUG
-		c = getopt_long(argc,argv,"hvd:H:fpPtiT",long_options,NULL);
+		c = getopt_long(argc,argv,"hvad:H:fnlpPtiT",long_options,NULL);
 #else
-		c = getopt_long(argc,argv,"hvd:H:f",long_options,NULL);
+		c = getopt_long(argc,argv,"hvad:H:fnl",long_options,NULL);
 #endif
 		if(c==-1)
 			break;
@@ -233,13 +366,19 @@ int main(int argc,char **argv)
 			printf(usage,progname);
 			printf("\t -h --help\t\tdisplay this message\n");
 			printf("\t -v --version\t\tdisplay version\n");
+			printf("\t -a --analyse\t\tanalyse raw_codes config files\n");
 			printf("\t -f --force\t\tforce raw mode\n");
+			printf("\t -n --disable-namespace\t\tdisables namespace checks\n");
+			printf("\t -l --list-namespace\t\tlist valid button names\n");
 			printf("\t -H --driver=driver\tuse given driver\n");
 			printf("\t -d --device=device\tread from given device\n");
 			exit(EXIT_SUCCESS);
 		case 'v':
 			printf("irrecord %s\n",IRRECORD_VERSION);
 			exit(EXIT_SUCCESS);
+		case 'a':
+			analyse = 1;
+			break;
 		case 'H':
 			if(hw_choose_driver(optarg) != 0){
 				fprintf(stderr, "Driver `%s' not supported.\n",
@@ -254,6 +393,13 @@ int main(int argc,char **argv)
 		case 'f':
 			force=1;
 			break;
+		case 'n':
+			disable_namespace = 1;
+			break;
+		case 'l':
+			fprint_namespace(stdout);
+			exit(EXIT_SUCCESS);
+			break;
 #ifdef DEBUG
 		case 'p':
 			get_pre=1;
@@ -311,7 +457,7 @@ int main(int argc,char **argv)
 				progname, filename, progname);
 			exit(EXIT_FAILURE);
 		}
-		remotes=read_config(fin);
+		remotes=read_config(fin, filename);
 		fclose(fin);
 		if(remotes==(void *) -1 || remotes==NULL)
 		{
@@ -320,6 +466,13 @@ int main(int argc,char **argv)
 				"data\n",progname,filename);
 			exit(EXIT_FAILURE);
 		}
+		if(analyse)
+		{
+			hw = hw_emulation;
+			for_each_remote(remotes, analyse_remote);
+			return EXIT_SUCCESS;
+		}
+		using_template = 1;
 #ifdef DEBUG
 		if(test)
 		{
@@ -340,8 +493,6 @@ int main(int argc,char **argv)
 		remote.codes=NULL;
 		remote.last_code=NULL;
 		remote.next=NULL;
-		remote.toggle_bit=0;
-		remote.toggle_bit_mask=0;
 		if(remote.pre_p==0 && remote.pre_s==0 &&
 		   remote.post_p==0 && remote.post_s==0)
 		{
@@ -367,6 +518,15 @@ int main(int argc,char **argv)
 		strcat(filename_new, ".conf");
 		filename = filename_new;
 	}
+	else
+	{
+		if(analyse)
+		{
+			fprintf(stderr, "%s: no input file given, "
+				"ignoring analyse flag\n", progname);
+			analyse = 0;
+		}
+	}
 	fout=fopen(filename,"w");
 	if(fout==NULL)
 	{
@@ -394,6 +554,7 @@ int main(int argc,char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
+	aeps = (hw.resolution>aeps ? hw.resolution:aeps);
 	
 	if(hw.rec_mode==LIRC_MODE_STRING)
 	{
@@ -428,7 +589,17 @@ int main(int argc,char **argv)
 	printf(
 "This program will record the signals from your remote control\n"
 "and create a config file for lircd.\n\n"
-"\n"
+"\n");
+	if(hw.name && strcmp(hw.name, "devinput") == 0)
+	{
+		printf(
+"Usually it's not necessary to create a new config file for devinput\n"
+"devices. A generic config file can be found at:\n"
+"http://www.lirc.org/remotes/devinput/\n"
+"You should try this config file before creating your own config file.\n"
+"\n");
+	}
+	printf(
 "A proper config file for lircd is maybe the most vital part of this\n"
 "package, so you should invest some time to create a working config\n"
 "file. Although I put a good deal of effort in this program it is often\n"
@@ -460,7 +631,7 @@ int main(int argc,char **argv)
 	switch(hw.rec_mode)
 	{
 	case LIRC_MODE_MODE2:
-		if(remotes==NULL && !get_lengths(&remote,force))
+		if(!using_template && !get_lengths(&remote, force, 1))
 		{
 			if(remote.gap==0)
 			{
@@ -473,8 +644,8 @@ int main(int argc,char **argv)
 			}
 			printf("Creating config file in raw mode.\n");
 			set_protocol(&remote, RAW_CODES);
-			remote.eps=EPS;
-			remote.aeps=AEPS;
+			remote.eps=eps;
+			remote.aeps=aeps;
 			break;
 		}
 		
@@ -494,7 +665,7 @@ int main(int argc,char **argv)
 	case LIRC_MODE_LIRCCODE:
 		if(hw.rec_mode==LIRC_MODE_CODE) remote.bits=CHAR_BIT;
 		else remote.bits=hw.code_length;
-		if(!get_gap_length(&remote))
+		if(!using_template && !get_gap_length(&remote))
 		{
 			fprintf(stderr,"%s: gap not found,"
 				" can't continue\n",progname);
@@ -506,7 +677,7 @@ int main(int argc,char **argv)
 		break;
 	}
 	
-	if(is_rc6(&remote))
+	if(!using_template && is_rc6(&remote))
 	{
 		sleep(1);
 		while(availabledata())
@@ -533,6 +704,15 @@ int main(int argc,char **argv)
 		char buffer[BUTTON];
 		char *string;
 
+		if(no_data)
+		{
+			fprintf(stderr,"%s: no data for 10 secs,"
+				" aborting\n",progname);
+			printf("The last button did not seem to generate any signal.\n");
+			printf("Press RETURN to continue.\n\n");
+			getchar();
+			no_data = 0;
+		}
 		printf("\nPlease enter the name for the next button (press <ENTER> to finish recording)\n");
 		string=fgets(buffer,BUTTON,stdin);
 		
@@ -560,6 +740,17 @@ int main(int argc,char **argv)
 		{
 			break;
 		}
+		if(!disable_namespace && !is_in_namespace(buffer))
+		{
+			printf("'%s' is not in name space (use "
+			       "--disable-namespace to disable checks)\n",
+			       buffer);
+			printf("Use '%s --list-namespace' to see a full list "
+			       "of valid button names\n",
+			       progname);
+			printf("Please try again.\n");
+			continue;
+		}
 		
 		if(is_raw(&remote))
 		{
@@ -592,9 +783,7 @@ int main(int argc,char **argv)
 				{
 					if(count==0)
 					{
-						fprintf(stderr,"%s: no data for 10 secs,"
-							" aborting\n",progname);
-						retval=EXIT_FAILURE;
+						no_data = 1;
 						break;
 					}
 					data=remote.gap;
@@ -665,9 +854,7 @@ int main(int argc,char **argv)
 
 			if(!waitfordata(10000000))
 			{
-				fprintf(stderr,"%s: no data for 10 secs,"
-					" aborting\n",progname);
-				retval=EXIT_FAILURE;
+				no_data = 1;
 				break;
 			}
 			last_remote=NULL;
@@ -748,7 +935,7 @@ int main(int argc,char **argv)
 		if(hw.deinit_func) hw.deinit_func();
 		exit(EXIT_FAILURE);
 	}
-	remotes=read_config(fin);
+	remotes=read_config(fin,filename);
 	fclose(fin);
 	if(remotes==NULL)
 	{
@@ -769,7 +956,9 @@ int main(int argc,char **argv)
 	
 	if(!has_toggle_bit_mask(remotes))
 	{
-		get_toggle_bit_mask(remotes);
+		if(!using_template &&
+		   strcmp(hw.name, "devinput") != 0)
+			get_toggle_bit_mask(remotes);
 	}
 	else
 	{
@@ -1208,7 +1397,6 @@ void get_post_data(struct ir_remote *remote)
 	}
 }
 
-#ifdef DEBUG
 void for_each_remote(struct ir_remote *remotes, remote_func func)
 {
 	struct ir_remote *remote;
@@ -1221,6 +1409,120 @@ void for_each_remote(struct ir_remote *remotes, remote_func func)
 	}
 }
 
+void analyse_remote(struct ir_remote *raw_data)
+{
+	struct ir_ncode *codes;
+	ir_code pre, code, code2, post;
+	int repeat_flag;
+	lirc_t min_remaining_gap, max_remaining_gap;
+	struct ir_ncode *new_codes;
+	size_t new_codes_count = 100;
+	int new_index = 0;
+	int ret;
+
+	if(!is_raw(raw_data))
+	{
+		fprintf(stderr, "%s: remote %s not in raw mode, ignoring\n",
+			progname, raw_data->name);
+		return;
+	}
+	aeps = raw_data->aeps;
+	eps = raw_data->eps;
+	emulation_data = raw_data;
+	next_code = NULL;
+	current_code = NULL;
+	current_index = 0;
+	memset(&remote, 0, sizeof(remote));
+	get_lengths(&remote, 0, 0 /* not interactive */ );
+
+	if(is_rc6(&remote) && remote.bits >= 5)
+	{
+		/* have to assume something as it's very difficult to
+		   extract the rc6_mask from the data that we have */
+		remote.rc6_mask = ((ir_code) 0x1ll) << (remote.bits-5);
+	}
+
+	remote.name = raw_data->name;
+	remote.freq = raw_data->freq;
+	
+	new_codes = malloc(new_codes_count * sizeof(*new_codes));
+	if(new_codes == NULL)
+	{
+		fprintf(stderr, "%s: out of memory\n",
+			progname);
+		return;
+	}
+	memset(new_codes, 0 , new_codes_count * sizeof(*new_codes));
+	codes = raw_data->codes;
+	while(codes->name!=NULL)
+	{
+		//printf("decoding %s\n", codes->name);
+		current_code = NULL;
+		current_index = 0;
+		next_code = codes;
+		
+                init_rec_buffer();
+		
+		ret = receive_decode(&remote,
+				     &pre, &code, &post,
+				     &repeat_flag,
+				     &min_remaining_gap,
+				     &max_remaining_gap);
+		if(!ret)
+		{
+			fprintf(stderr, "%s: decoding of %s failed\n",
+				progname, codes->name);
+		}
+		else
+		{
+			if(new_index+1 >= new_codes_count)
+			{
+				struct ir_ncode *renew_codes;
+				
+				new_codes_count *= 2;
+				renew_codes = realloc
+					(new_codes,
+					 new_codes_count * sizeof(*new_codes));
+				if(renew_codes == NULL)
+				{
+					fprintf(stderr, "%s: out of memory\n",
+						progname);
+					free(new_codes);
+					return;
+				}
+				memset(&new_codes[new_codes_count/2], 0 , new_codes_count/2 * sizeof(*new_codes));
+				new_codes = renew_codes;
+			}
+			
+			clear_rec_buffer();
+			ret = receive_decode(&remote,
+					     &pre, &code2, &post,
+					     &repeat_flag,
+					     &min_remaining_gap,
+					     &max_remaining_gap);
+			if(ret && code2 != code)
+			{
+				new_codes[new_index].next = malloc(sizeof(*(new_codes[new_index].next)));
+				if(new_codes[new_index].next)
+				{
+					memset(new_codes[new_index].next, 0, sizeof(*(new_codes[new_index].next)));
+					new_codes[new_index].next->code = code2;
+				}
+			}
+			new_codes[new_index].name = codes->name;
+			new_codes[new_index].code = code;			
+			new_index++;
+		}
+		codes++;
+	}
+	new_codes[new_index].name = NULL;
+	remote.codes = new_codes;
+	fprint_remotes(stdout, &remote);
+	remote.codes = NULL;
+	free(new_codes);
+}
+
+#ifdef DEBUG
 void remove_pre_data(struct ir_remote *remote)
 {
 	struct ir_ncode *codes;
@@ -1387,28 +1689,34 @@ inline lirc_t calc_signal(struct lengths *len)
 	return((lirc_t) (len->sum/len->count));
 }
 
-int get_lengths(struct ir_remote *remote,int force)
+int get_lengths(struct ir_remote *remote, int force, int interactive)
 {
 	int retval;
-	lirc_t data,average,sum,remaining_gap,header;
+	lirc_t data,average,maxspace,sum,remaining_gap,header;
 	enum analyse_mode mode=MODE_GAP;
 	int first_signal;
 
-	printf("Now start pressing buttons on your remote control.\n\n");
-	printf(
+	if(interactive)
+	{
+		printf("Now start pressing buttons on your remote control.\n\n");
+		printf(
 "It is very important that you press many different buttons and hold them\n"
 "down for approximately one second. Each button should generate at least one\n"
 "dot but in no case more than ten dots of output.\n"
 "Don't stop pressing buttons until two lines of dots (2x80) have been\n"
 "generated.\n\n");
-	printf("Press RETURN now to start recording.");fflush(stdout);
-	getchar();
-	flushhw();
+		printf("Press RETURN now to start recording.");fflush(stdout);
+		getchar();
+		flushhw();
+	}
 	retval=1;
-	average=0;sum=0;count=0;count_spaces=0;
+	average=0;maxspace=0;sum=0;count=0;count_spaces=0;
 	count_3repeats=0;count_5repeats=0;count_signals=0;
 	first_signal=-1;header=0;
 	first_length=0;
+	first_lengths = 0;
+	second_lengths = 0;
+	memset(lengths, 0, sizeof(lengths));
 	while(1)
 	{
 		data=hw.readdata(10000000);
@@ -1431,12 +1739,15 @@ int get_lengths(struct ir_remote *remote,int force)
 					continue;
 				}
 				average=data;
+				maxspace=data;
 			}
 			else if(is_space(data))
 			{
 				if(data>MIN_GAP || data>100*average ||
 				   /* this MUST be a gap */
-				   (count_spaces>10 && data>5*average))  
+				   (count_spaces>10 && data>5*maxspace/2)
+				   /* || Echostar
+				      (count_spaces>20 && data>9*maxspace/10)*/)
 					/* this should be a gap */
 				{
 					struct lengths *scan;
@@ -1448,7 +1759,7 @@ int get_lengths(struct ir_remote *remote,int force)
 					merge_lengths(first_sum);
 					add_length(&first_gap,data);
 					merge_lengths(first_gap);
-					sum=0;count_spaces=0;average=0;
+					sum=0;count_spaces=0;average=0;maxspace=0;
 					
 					maxcount=0;
 					scan=first_sum;
@@ -1460,7 +1771,7 @@ int get_lengths(struct ir_remote *remote,int force)
 						{
 							remote->gap=calc_signal(scan);
 							remote->flags|=CONST_LENGTH;
-							printf("\nFound const length: %lu\n",(unsigned long) remote->gap);
+							iprintf(interactive, "\nFound const length: %lu\n",(unsigned long) remote->gap);
 							break;
 						}
 						scan=scan->next;
@@ -1475,7 +1786,7 @@ int get_lengths(struct ir_remote *remote,int force)
 							if(scan->count>SAMPLES)
 							{
 								remote->gap=calc_signal(scan);
-								printf("\nFound gap: %lu\n",(unsigned long) remote->gap);
+								iprintf(interactive, "\nFound gap: %lu\n",(unsigned long) remote->gap);
 								break;
 							}
 							scan=scan->next;
@@ -1483,7 +1794,7 @@ int get_lengths(struct ir_remote *remote,int force)
 					}
 					if(scan!=NULL)
 					{
-						printf("Please keep on pressing buttons like described above.\n");
+						iprintf(interactive, "Please keep on pressing buttons like described above.\n");
 						mode=MODE_HAVE_GAP;
 						sum=0;
 						count=0;
@@ -1499,10 +1810,13 @@ int get_lengths(struct ir_remote *remote,int force)
 						continue;
 					}
 					
-					for(i=maxcount-lastmaxcount;i>0;i--)
+					if(interactive)
 					{
-						printf(".");
-						fflush(stdout);
+						for(i=maxcount-lastmaxcount;i>0;i--)
+						{
+							printf(".");
+							fflush(stdout);
+						}
 					}
 					lastmaxcount=maxcount;
 					
@@ -1511,6 +1825,10 @@ int get_lengths(struct ir_remote *remote,int force)
 				average=(average*count_spaces+data)
 				/(count_spaces+1);
 				count_spaces++;
+				if(data>maxspace)
+				{
+					maxspace=data;
+				}
 			}
 			if(count>SAMPLES*MAX_SIGNALS*2)
 			{
@@ -1544,8 +1862,9 @@ int get_lengths(struct ir_remote *remote,int force)
 			}
 			sum+=data&PULSE_MASK;
 
-			if((data&PULSE_MASK)>=remaining_gap*(100-EPS)/100
-			   || (data&PULSE_MASK)>=remaining_gap-AEPS)
+			if(count>2 &&
+			   ((data&PULSE_MASK)>=remaining_gap*(100-eps)/100
+			    || (data&PULSE_MASK)>=remaining_gap-aeps))
 			{
 				if(is_space(data))
 				{
@@ -1582,7 +1901,10 @@ int get_lengths(struct ir_remote *remote,int force)
 					{
 						int i;
 
-						printf(".");fflush(stdout);
+						if(interactive)
+						{
+							printf(".");fflush(stdout);
+						}
 						count_signals++;
 						add_length(&first_1lead,signals[0]);
 						merge_lengths(first_1lead);
@@ -1646,42 +1968,44 @@ int get_lengths(struct ir_remote *remote,int force)
 #endif
 				if(count_signals>=SAMPLES)
 				{
-					printf("\n");
-					get_scheme(remote);
-					if(!get_header_length(remote) ||
-					   !get_trail_length(remote) ||
-					   !get_lead_length(remote) ||
-					   !get_repeat_length(remote) ||
-					   !get_data_length(remote))
+					iprintf(interactive, "\n");
+					get_scheme(remote, interactive);
+					if(!get_header_length(remote, interactive) ||
+					   !get_trail_length(remote, interactive) ||
+					   !get_lead_length(remote, interactive) ||
+					   !get_repeat_length(remote, interactive) ||
+					   !get_data_length(remote, interactive))
 					{
 						retval=0;
 					}
 					break;
 				}
-				if((data&PULSE_MASK)<=(remaining_gap+header)*(100+EPS)/100
-				   || (data&PULSE_MASK)<=(remaining_gap+header)+AEPS)
+				if((data&PULSE_MASK)<=(remaining_gap+header)*(100+eps)/100
+				   || (data&PULSE_MASK)<=(remaining_gap+header)+aeps)
 				{
 					first_signal=0;
 					header=0;
 				}
 				else
+				{
 					first_signal=1;
+				}
 			}
 		}
 	}
-	free_lengths(first_space);
-	free_lengths(first_pulse);
-	free_lengths(first_sum);
-	free_lengths(first_gap);
-	free_lengths(first_repeat_gap);
-	free_lengths(first_signal_length);
-	free_lengths(first_headerp);
-	free_lengths(first_headers);
-	free_lengths(first_1lead);
-	free_lengths(first_3lead);
-	free_lengths(first_trail);
-	free_lengths(first_repeatp);
-	free_lengths(first_repeats);
+	free_lengths(&first_space);
+	free_lengths(&first_pulse);
+	free_lengths(&first_sum);
+	free_lengths(&first_gap);
+	free_lengths(&first_repeat_gap);
+	free_lengths(&first_signal_length);
+	free_lengths(&first_headerp);
+	free_lengths(&first_headers);
+	free_lengths(&first_1lead);
+	free_lengths(&first_3lead);
+	free_lengths(&first_trail);
+	free_lengths(&first_repeatp);
+	free_lengths(&first_repeats);
 	return(retval);
 }
 
@@ -1731,17 +2055,19 @@ int add_length(struct lengths **first,lirc_t length)
 	return(1);
 }
 
-void free_lengths(struct lengths *first)
+void free_lengths(struct lengths **firstp)
 {
-	struct lengths *next;
+	struct lengths *first, *next;
 
-	if(first==NULL) return;
+	first = *firstp;
+	if(first == NULL) return;
 	while(first!=NULL)
 	{
-		next=first->next;
+		next = first->next;
 		free(first);
 		first=next;
 	}
+	*firstp = NULL;
 }
 
 void merge_lengths(struct lengths *first)
@@ -1760,15 +2086,15 @@ void merge_lengths(struct lengths *first)
 			new_sum=l->sum+inner->sum;
 			new_count=l->count+inner->count;
 			
-			if((l->max<=new_sum/new_count+AEPS &&
-			    l->min>=new_sum/new_count-AEPS &&
-			    inner->max<=new_sum/new_count+AEPS &&
-			    inner->min>=new_sum/new_count-AEPS)
+			if((l->max<=new_sum/new_count+aeps &&
+			    l->min+aeps>=new_sum/new_count &&
+			    inner->max<=new_sum/new_count+aeps &&
+			    inner->min+aeps>=new_sum/new_count)
 			   ||
-			   (l->max<=new_sum/new_count*(100+EPS) &&
-			    l->min>=new_sum/new_count*(100-EPS) &&
-			    inner->max<=new_sum/new_count*(100+EPS) &&
-			    inner->min>=new_sum/new_count*(100-EPS)))
+			   (l->max<=new_sum/new_count*(100+eps) &&
+			    l->min>=new_sum/new_count*(100-eps) &&
+			    inner->max<=new_sum/new_count*(100+eps) &&
+			    inner->min>=new_sum/new_count*(100-eps)))
 			{
 				l->sum=new_sum;
 				l->count=new_count;
@@ -1801,7 +2127,7 @@ void merge_lengths(struct lengths *first)
 #       endif
 }
 
-void get_scheme(struct ir_remote *remote)
+void get_scheme(struct ir_remote *remote, int interactive)
 {
 	unsigned int i,length=0,sum=0;
 
@@ -1821,11 +2147,12 @@ void get_scheme(struct ir_remote *remote)
 	       "first_lengths: %lu second_lengths: %lu\n",
 	       sum,length+1,lengths[length],first_lengths,second_lengths);
 #       endif
+	/* FIXME !!! this heuristic is too bad */
 	if(lengths[length]>=TH_SPACE_ENC*sum/100)
 	{
 		length++;
-		printf("Space/pulse encoded remote control found.\n");
-		printf("Signal length is %u.\n",length);
+		iprintf(interactive, "Space/pulse encoded remote control found.\n");
+		iprintf(interactive, "Signal length is %u.\n",length);
 		/* this is not yet the
 		   number of bits */
 		remote->bits=length;
@@ -1862,17 +2189,18 @@ void get_scheme(struct ir_remote *remote)
 				
 				maxs=get_max_length(first_space,NULL);
 				
-				if((calc_signal(maxp)<500 ||
-				    calc_signal(max2p)<500) &&
-				   (calc_signal(maxs)<500 ||
-				    calc_signal(max2s)<500))
+				if(length > 20 &&
+				   (calc_signal(maxp)<TH_RC6_SIGNAL ||
+				    calc_signal(max2p)<TH_RC6_SIGNAL) &&
+				   (calc_signal(maxs)<TH_RC6_SIGNAL ||
+				    calc_signal(max2s)<TH_RC6_SIGNAL))
 				{
-					printf("RC-6 remote control found.\n");
+					iprintf(interactive, "RC-6 remote control found.\n");
 					set_protocol(remote, RC6);
 				}
 				else
 				{
-					printf("RC-5 remote control found.\n");
+					iprintf(interactive, "RC-5 remote control found.\n");
 					set_protocol(remote, RC5);
 				}
 				return;
@@ -1880,7 +2208,7 @@ void get_scheme(struct ir_remote *remote)
 		}
 	}
 	length++;
-	printf("Suspicious data length: %u.\n",length);
+	iprintf(interactive, "Suspicious data length: %u.\n",length);
 	/* this is not yet the number of bits */
 	remote->bits=length;
 	set_protocol(remote, SPACE_ENC);
@@ -1895,6 +2223,10 @@ struct lengths *get_max_length(struct lengths *first,unsigned int *sump)
 	max_length=first;
 	sum=first->count;
 	
+#       ifdef DEBUG
+	if(first->count>0) printf("%u x %lu\n", first->count,
+				  (unsigned long) calc_signal(first));
+#       endif
 	scan=first->next;
 	while(scan)
 	{
@@ -1913,7 +2245,7 @@ struct lengths *get_max_length(struct lengths *first,unsigned int *sump)
 	return(max_length);
 }
 
-int get_trail_length(struct ir_remote *remote)
+int get_trail_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	struct lengths *max_length;
@@ -1927,22 +2259,22 @@ int get_trail_length(struct ir_remote *remote)
 #       endif
 	if(max_count>=sum*TH_TRAIL/100)
 	{
-		printf("Found trail pulse: %lu\n",
-		       (unsigned long) calc_signal(max_length));
+		iprintf(interactive, "Found trail pulse: %lu\n",
+			(unsigned long) calc_signal(max_length));
 		remote->ptrail=calc_signal(max_length);
 		return(1);
 	}
-	printf("No trail pulse found.\n");
+	iprintf(interactive, "No trail pulse found.\n");
 	return(1);
 }
 
-int get_lead_length(struct ir_remote *remote)
+int get_lead_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	struct lengths *first_lead,*max_length,*max2_length;
 	lirc_t a,b,swap;
 	
-	if(!is_biphase(remote)) return(1);
+	if(!is_biphase(remote) || has_header(remote)) return(1);
 	if(is_rc6(remote)) return(1);
 	
 	first_lead=has_header(remote) ? first_3lead:first_1lead;
@@ -1953,7 +2285,7 @@ int get_lead_length(struct ir_remote *remote)
 #       endif
 	if(max_count>=sum*TH_LEAD/100)
 	{
-		printf("Found lead pulse: %lu\n",
+		iprintf(interactive, "Found lead pulse: %lu\n",
 		       (unsigned long) calc_signal(max_length));
 		remote->plead=calc_signal(max_length);
 		return(1);
@@ -1968,18 +2300,18 @@ int get_lead_length(struct ir_remote *remote)
 	{
 		swap=a; a=b; b=swap;
 	}
-	if(abs(2*a-b)<b*EPS/100 || abs(2*a-b)<AEPS)
+	if(abs(2*a-b)<b*eps/100 || abs(2*a-b)<aeps)
 	{
-		printf("Found hidden lead pulse: %lu\n",
+		iprintf(interactive, "Found hidden lead pulse: %lu\n",
 		       (unsigned long) a);
 		remote->plead=a;
 		return(1);
 	}
-	printf("No lead pulse found.\n");
+	iprintf(interactive, "No lead pulse found.\n");
 	return(1);
 }
 
-int get_header_length(struct ir_remote *remote)
+int get_header_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	lirc_t headerp,headers;
@@ -1992,7 +2324,7 @@ int get_header_length(struct ir_remote *remote)
 	}
 	else
 	{
-		printf("No header data.\n");
+		iprintf(interactive, "No header data.\n");
 		return(1);
 	}
 #       ifdef DEBUG
@@ -2012,24 +2344,24 @@ int get_header_length(struct ir_remote *remote)
 			headerp=calc_signal(max_plength);
 			headers=calc_signal(max_slength);
 
-			printf("Found possible header: %lu %lu\n",
-			       (unsigned long) headerp,
-			       (unsigned long) headers);
+			iprintf(interactive, "Found possible header: %lu %lu\n",
+				(unsigned long) headerp,
+				(unsigned long) headers);
 			remote->phead=headerp;
 			remote->shead=headers;
 			if(first_lengths<second_lengths)
 			{
-				printf("Header is not being repeated.\n");
+				iprintf(interactive, "Header is not being repeated.\n");
 				remote->flags|=NO_HEAD_REP;
 			}
 			return(1);
 		}
 	}
-	printf("No header found.\n");
+	iprintf(interactive, "No header found.\n");
 	return(1);
 }
 
-int get_repeat_length(struct ir_remote *remote)
+int get_repeat_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	lirc_t repeatp,repeats,repeat_gap;
@@ -2043,7 +2375,7 @@ int get_repeat_length(struct ir_remote *remote)
 			printf("Repeat inconsitentcy.\n");
 			return(0);
 		}
-		printf("No repeat code found.\n");
+		iprintf(interactive, "No repeat code found.\n");
 		return(1);
 	}
 
@@ -2078,9 +2410,9 @@ int get_repeat_length(struct ir_remote *remote)
 			repeatp=calc_signal(max_plength);
 			repeats=calc_signal(max_slength);
 			
-			printf("Found repeat code: %lu %lu\n",
-			       (unsigned long) repeatp,
-			       (unsigned long) repeats);
+			iprintf(interactive, "Found repeat code: %lu %lu\n",
+				(unsigned long) repeatp,
+				(unsigned long) repeats);
 			remote->prepeat=repeatp;
 			remote->srepeat=repeats;
 			if(!(remote->flags&CONST_LENGTH))
@@ -2088,15 +2420,15 @@ int get_repeat_length(struct ir_remote *remote)
 				max_slength=get_max_length(first_repeat_gap,
 							   NULL);
 				repeat_gap=calc_signal(max_slength);
-				printf("Found repeat gap: %lu\n",
-				       (unsigned long) repeat_gap);
+				iprintf(interactive, "Found repeat gap: %lu\n",
+					(unsigned long) repeat_gap);
 				remote->repeat_gap=repeat_gap;
 				
 			}
 			return(1);
 		}
 	}
-	printf("No repeat header found.\n");
+	iprintf(interactive, "No repeat header found.\n");
 	return(1);
 
 }
@@ -2130,7 +2462,7 @@ void unlink_length(struct lengths **first,struct lengths *remove)
 	printf("unlink_length(): report this bug!\n");
 }
 
-int get_data_length(struct ir_remote *remote)
+int get_data_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	lirc_t p1,p2,s1,s2;
@@ -2192,8 +2524,8 @@ int get_data_length(struct ir_remote *remote)
 #                       endif
 
 
-			remote->eps=EPS;
-			remote->aeps=AEPS;
+			remote->eps=eps;
+			remote->aeps=aeps;
 			if(is_biphase(remote))
 			{
 				if(max2_plength==NULL || max2_slength==NULL)
@@ -2201,7 +2533,7 @@ int get_data_length(struct ir_remote *remote)
 					printf("Unknown encoding found.\n");
 					return(0);
 				}
-				printf("Signals are biphase encoded.\n");
+				iprintf(interactive, "Signals are biphase encoded.\n");
 				p1=calc_signal(max_plength);
 				p2=calc_signal(max2_plength);
 				s1=calc_signal(max_slength);
@@ -2230,7 +2562,7 @@ int get_data_length(struct ir_remote *remote)
 				if(max2_plength)
 				{
 					p2=calc_signal(max2_plength);
-					printf("Signals are pulse encoded.\n");
+					iprintf(interactive, "Signals are pulse encoded.\n");
 					remote->pone=max(p1,p2);
 					remote->sone=s1;
 					remote->pzero=min(p1,p2);
@@ -2244,7 +2576,7 @@ int get_data_length(struct ir_remote *remote)
 				else
 				{
 					s2=calc_signal(max2_slength);
-					printf("Signals are space encoded.\n");
+					iprintf(interactive, "Signals are space encoded.\n");
 					remote->pone=p1;
 					remote->sone=max(s1,s2);
 					remote->pzero=p1;
@@ -2263,7 +2595,7 @@ int get_data_length(struct ir_remote *remote)
 				{
 					remote->phead=remote->shead=0;
 					remote->flags&=~NO_HEAD_REP;
-					printf("Removed header.\n");
+					iprintf(interactive, "Removed header.\n");
 				}
 				if(is_biphase(remote) &&
 				   expect(remote,remote->shead,remote->sone))
@@ -2271,7 +2603,7 @@ int get_data_length(struct ir_remote *remote)
 					remote->plead=remote->phead;
 					remote->phead=remote->shead=0;
 					remote->flags&=~NO_HEAD_REP;
-					printf("Removed header.\n");
+					iprintf(interactive, "Removed header.\n");
 				}
 			}
 			if(is_biphase(remote))
@@ -2298,12 +2630,12 @@ int get_data_length(struct ir_remote *remote)
 					      (has_header(remote) ? 2:0)+
 					      1-(remote->ptrail>0 ? 2:0))/2;
 			}
-			printf("Signal length is %d\n",remote->bits);
-			free_lengths(max_plength);
-			free_lengths(max_slength);
+			iprintf(interactive, "Signal length is %d\n",remote->bits);
+			free_lengths(&max_plength);
+			free_lengths(&max_slength);
 			return(1);
 		}
-		free_lengths(max_plength);
+		free_lengths(&max_plength);
 	}
 	printf("Could not find data lengths.\n");
 	return(0);
@@ -2318,8 +2650,8 @@ int get_gap_length(struct ir_remote *remote)
 	int maxcount,lastmaxcount;
 	lirc_t gap;
 	
-	remote->eps=EPS;
-	remote->aeps=AEPS;
+	remote->eps=eps;
+	remote->aeps=aeps;
 
 	count=0;flag=0;lastmaxcount=0;
 	printf("Hold down an arbitrary button.\n");
@@ -2331,7 +2663,7 @@ int get_gap_length(struct ir_remote *remote)
 		}
 		if(!waitfordata(10000000))
 		{
-			free_lengths(gaps);
+			free_lengths(&gaps);
 			return(0);
 		}
 		gettimeofday(&start,NULL);
@@ -2354,11 +2686,9 @@ int get_gap_length(struct ir_remote *remote)
 				if(scan->count>SAMPLES)
 				{
 					remote->gap=calc_signal(scan);
-					/* this does not work very reliably */
-					remote->gap+=100000;
 					printf("\nFound gap length: %lu\n",
 					       (unsigned long) remote->gap);
-					free_lengths(gaps);
+					free_lengths(&gaps);
 					return(1);
 				}
 				scan=scan->next;
